@@ -23,7 +23,7 @@ mod ws;
 #[doc(inline)]
 pub use axum_server::HttpConfig;
 /// A boxed HTTP body. This is used to unify response bodies.
-pub type BoxBody = http_body::combinators::BoxBody<Bytes, BoxError>;
+pub type BoxBody = http_body::combinators::UnsyncBoxBody<Bytes, BoxError>;
 /// A HTTP request.
 pub type HttpRequest = http::Request<hyper::Body>;
 /// A HTTP response.
@@ -32,7 +32,7 @@ pub type HttpResponse = http::Response<BoxBody>;
 /// Convert a body with the correct attributes to a [`BoxBody`].
 pub fn box_body<B>(body: B) -> BoxBody
 where
-    B: http_body::Body<Data = Bytes> + Send + Sync + 'static,
+    B: http_body::Body<Data = Bytes> + Send + 'static,
     B::Error: Into<BoxError>,
 {
     BoxBody::new(body.map_err(Into::into))
